@@ -65,6 +65,7 @@
 
 /* Implementational detail - not for global header */
 #define OLE_DEBUG 0
+#define OLE_CHAIN_DEBUG 0
 
 /* FIXME tenix add ADD_BBD_LIST_BLOCK where it should be used) */
 #define ADD_BBD_LIST_BLOCK   0xfffffffc       /* -4 */
@@ -331,7 +332,7 @@ write_cache_block (MsOle *f, BBBlkAttr *attr)
 	if (f->syswrap->lseek (f->file_des, offset, SEEK_SET)==(off_t)-1 ||
 	    f->syswrap->write (f->file_des, attr->data, BB_BLOCK_SIZE) == -1)
 		g_warning ("Fatal error writing block %d at %d\n", attr->blk, offset);
-#if OLE_DEBUG > 0
+#if OLE_DEBUG > 2
 	g_print ("Writing cache block %d to offset %d\n",
 		 attr->blk, offset);
 #endif	
@@ -380,7 +381,7 @@ get_block_ptr (MsOle *f, BLP b, gboolean forwrite)
 	g_assert (!attr->data);
 	if (min) {
 		g_assert (min->data);
-#if OLE_DEBUG > 0
+#if OLE_DEBUG > 2
 		g_print ("Replacing cache block %d with %d\n", min->blk, b);
 #endif
 		if (min->dirty)
@@ -611,6 +612,10 @@ void
 ms_ole_debug (MsOle *fs, int magic)
 {
 	switch (magic) {
+	case 0:
+		dump_allocation (fs);
+	case 1:
+		dump_header (fs);
 	case 2:
 		if (fs->pps)
 			dump_tree (fs->pps, 0);
@@ -618,8 +623,6 @@ ms_ole_debug (MsOle *fs, int magic)
 			g_print ("There are no tree (no pps)\n");
 		break;
 	default:
-		dump_header (fs);
-		dump_allocation (fs);
 		break;
 	}
 }
@@ -1294,6 +1297,29 @@ pps_encode_tree_chain (MsOle *f, GList *list)
 }
 
 
+static void
+destroy_pps (GList *l)
+{
+	GList *tmp;
+
+	if (!l)
+		return;
+
+	for (tmp = l; tmp; tmp = g_list_next (tmp)) {
+		PPS *pps = tmp->data;
+		if (pps->name) {
+			g_free (pps->name);
+			pps->name = NULL;
+		}
+		destroy_pps (pps->children);
+		pps->children = NULL;
+		g_free (pps);
+		pps = NULL;
+	}
+	g_list_free (l);
+}
+
+
 static int
 write_pps (MsOle *f)
 {
@@ -1324,7 +1350,8 @@ write_pps (MsOle *f)
 	pps_encode_tree_initial (f, f->pps, &idx);
 	pps_encode_tree_chain   (f, f->pps);
 
-	f->pps = 0;
+	destroy_pps (f->pps);
+	f->pps = NULL;
 	f->num_pps = 0;
 	return 1;
 }
@@ -1600,7 +1627,9 @@ ms_ole_open_vfs (MsOle **fs, const char *name,
 	if (file == -1) {
 		f->file_des = file = f->syswrap->open2 (name, O_RDONLY);
 		f->mode = 'r';
+#ifdef HAVE_MMAP
 		prot &= ~PROT_WRITE;
+#endif
 	}
 
 	if ((file == -1) || !(f->syswrap->isregfile (file))) {
@@ -1822,29 +1851,6 @@ ms_ole_create_vfs (MsOle **fs, const char *name, gboolean try_mmap,
 }
 
 
-static void
-destroy_pps (GList *l)
-{
-	GList *tmp;
-
-	if (!l)
-		return;
-
-	for (tmp = l; tmp; tmp = g_list_next (tmp)) {
-		PPS *pps = tmp->data;
-		if (pps->name) {
-			g_free (pps->name);
-			pps->name = NULL;
-		}
-		destroy_pps (pps->children);
-		pps->children = NULL;
-		g_free (pps);
-		pps = NULL;
-	}
-	g_list_free (l);
-}
-
-
 /**
  * ms_ole_destroy:
  * @fs: filesystem object.
@@ -2330,12 +2336,12 @@ ms_ole_append_block (MsOleStream *s)
 
 		if (lastblk != END_OF_CHAIN) { /* Link onwards */
 			g_array_index (s->file->sb, BLP, lastblk) = block;
-#if OLE_DEBUG > 1
+#if OLE_CHAIN_DEBUG > 1
 			g_print ("Chained Small block %d to previous block %d\n", block, lastblk);
 #endif
 		} else { /* First block in a file */
 			PPS *p = s->pps;
-#if OLE_DEBUG > 0
+#if OLE_CHAIN_DEBUG > 0
 			g_print ("Set first Small block to %d\n", block);
 #endif
 			p->start = block;
@@ -2349,7 +2355,7 @@ ms_ole_append_block (MsOleStream *s)
 			lastblk = ms_array_index (s->blocks, BLP, s->blocks->len-1);
 
 		block = next_free_bb (s->file);
-#if OLE_DEBUG > 0
+#if OLE_CHAIN_DEBUG > 0
 		{
 			int lp;
 			g_assert (g_array_index (s->file->bb, BLP, block) == UNUSED_BLOCK);
@@ -2361,12 +2367,12 @@ ms_ole_append_block (MsOleStream *s)
 
 		if (lastblk != END_OF_CHAIN) { /* Link onwards */
 			g_array_index (s->file->bb, BLP, lastblk) = block;
-#if OLE_DEBUG > 1
+#if OLE_CHAIN_DEBUG > 1
 		        g_print ("Chained Big block %d to block %d\n", block, lastblk);
 #endif
 		} else { /* First block in a file */
 			PPS *p = s->pps;
-#if OLE_DEBUG > 0
+#if OLE_CHAIN_DEBUG > 0
 			g_print ("Set first Big block to %d\n", block);
 #endif
 			p->start = block;
@@ -2402,7 +2408,7 @@ ms_ole_write_bb (MsOleStream *s, guint8 *ptr, MsOlePos length)
 		
 		dest = BB_W_PTR(s->file, block) + offset;
 
-#if OLE_DEBUG > 1
+#if OLE_DEBUG > 2
 		g_print ("Copy %d bytes to block %d\n", cpylen, block);
 #endif
 		memcpy (dest, ptr, cpylen);
@@ -2841,6 +2847,9 @@ ms_ole_stream_open (MsOleStream ** const stream, MsOle *f,
 	    MS_OLE_ERR_OK)
 		return result;
 
+	if (p->type != MsOleStreamT)
+		return MS_OLE_ERR_INVALID;
+
 	s           = g_new0 (MsOleStream, 1);
 	s->file     = f;
 	s->pps      = p;
@@ -2861,10 +2870,10 @@ ms_ole_stream_open (MsOleStream ** const stream, MsOle *f,
 		s->write     = ms_ole_write_bb;
 
 		s->blocks    = g_array_new (FALSE, FALSE, sizeof(BLP));
-		s->type   = MsOleLargeBlock;
+		s->type      = MsOleLargeBlock;
 		for (lp = 0; !panic & (lp < (s->size + BB_BLOCK_SIZE - 1) / BB_BLOCK_SIZE); lp++) {
 			g_array_append_val (s->blocks, b);
-#if OLE_DEBUG > 1
+#if OLE_DEBUG > 2
 			g_print ("Block %d\n", b);
 #endif
 			if (b == END_OF_CHAIN ||
@@ -2912,7 +2921,7 @@ ms_ole_stream_open (MsOleStream ** const stream, MsOle *f,
 		else
 			s->blocks = NULL;
 
-		s->type   = MsOleSmallBlock;
+		s->type = MsOleSmallBlock;
 
 		for (lp = 0; !panic & (lp < (s->size + SB_BLOCK_SIZE - 1) / SB_BLOCK_SIZE); lp++) {
 			g_array_append_val (s->blocks, b);
@@ -2969,7 +2978,7 @@ ms_ole_stream_open (MsOleStream ** const stream, MsOle *f,
  * Return value: a #MsOleErr code.
  **/
 MsOleErr
-ms_ole_stream_duplicate (MsOleStream **s, const MsOleStream * const stream)
+ms_ole_stream_duplicate (MsOleStream ** const s, const MsOleStream * const stream)
 {
 	if (!s || !stream)
 		return MS_OLE_ERR_BADARG;
@@ -2996,7 +3005,7 @@ ms_ole_stream_duplicate (MsOleStream **s, const MsOleStream * const stream)
  * Return value: a #MsOleErr code.
  **/
 MsOleErr
-ms_ole_stream_close (MsOleStream **s)
+ms_ole_stream_close (MsOleStream ** const s)
 {
 	if (*s) {
 		if ((*s)->file && (*s)->file->mode == 'w')
