@@ -873,6 +873,121 @@ dump_vba (MsOle *f)
 	g_free (txt);
 }
 
+static void
+hammer_stream (MsOleStream *stream, gboolean write)
+{
+	int     i, lp;
+	guint8 *mem;
+
+#define BLOCK_LEN  1000
+#define NUM_BLOCKS 7100
+
+	for (lp = 0; lp < NUM_BLOCKS; lp++) {
+
+		mem = g_malloc (BLOCK_LEN);
+
+		if (!write)
+			g_assert (stream->read_copy (
+				stream, mem, BLOCK_LEN));
+
+		for (i = 0; i < BLOCK_LEN; i++) {
+			guint8 sig;
+
+			sig = 0; //(i << 3) ^ (lp);
+			if (write)
+				mem [i] = sig;
+			else {
+				if (i == 0 && lp == 0) {
+					/* Ok, this dump shows how:
+
+					   Old value = 0 '\000'
+					   New value = 154 '\232'
+					   0x4001bf17 in write_bb (f=0x804e638) at ms-ole.c:916
+					   916			SET_BBD_LIST (f, lp, blk);
+					   (gdb) bt
+					   #0  0x4001bf17 in write_bb (f=0x804e638) at ms-ole.c:916
+					   #1  0x4001e55f in ms_ole_cleanup (f=0x804e638) at ms-ole.c:1538
+					   #2  0x4001f349 in ms_ole_destroy (ptr=0xbffff6f4) at ms-ole.c:1873
+					   #3  0x0804b757 in do_regression_tests () at test-ole.c:953
+					   #4  0x0804b9ad in main (argc=2, argv=0xbffff78c) at test-ole.c:985
+
+					   Corrupt the data in the stream - libole2 does not work with
+					   large files.
+					 */
+					ms_ole_dump (mem, 64);
+				}
+				if (i < 32 && lp == 0)
+					continue;
+
+				if (mem [i] != sig)
+					g_error ("Mismatch 0x%x 0x%x", mem [i], sig);
+			}
+		}
+
+		if (write)
+			stream->write (stream, mem, BLOCK_LEN);
+	}
+}
+
+#define REG_NAME "/tmp/ole-file"
+#define STREAM_NAME "foo"
+
+static void
+do_regression_tests (void)
+{
+	MsOle       *ole;
+	MsOleStream *stream;
+	MsOleStat    stat;
+
+	if (ms_ole_create_vfs (&ole, REG_NAME, TRUE, NULL))
+		g_error ("Couldn't open '" REG_NAME "'");
+
+	ms_ole_debug (ole, 1);
+
+	g_print ("\n\nwrite tests\n");
+	g_assert (!ms_ole_stream_open (&stream, ole, "/", STREAM_NAME, 'w'));
+	hammer_stream (stream, TRUE);
+	g_assert (!ms_ole_stream_close (&stream));
+
+	ms_ole_debug (ole, 1);
+
+	g_print ("\n\ncheck attrs.\n");
+	g_assert (!ms_ole_stat (&stat, ole, "/", STREAM_NAME));
+	g_assert (stat.type == MsOleStreamT);
+	g_assert (stat.size == BLOCK_LEN * NUM_BLOCKS);
+
+	g_print ("\n\nread tests\n");
+	g_assert (!ms_ole_stream_open (&stream, ole, "/", STREAM_NAME, 'r'));
+	hammer_stream (stream, FALSE);
+	g_assert (!ms_ole_stream_close (&stream));
+
+	g_print ("\n\ncheck attrs.\n");
+	g_assert (!ms_ole_stat (&stat, ole, "/", STREAM_NAME));
+	g_assert (stat.type == MsOleStreamT);
+	g_assert (stat.size == BLOCK_LEN * NUM_BLOCKS);
+
+	g_print ("\n\nclosing\n");
+	ms_ole_destroy (&ole);
+
+	g_print ("\n\nre-opening\n");
+	g_assert (!ms_ole_open_vfs (&ole, REG_NAME, TRUE, NULL));
+	ms_ole_debug (ole, 1);
+
+	g_print ("\n\ncheck attrs.\n");
+	g_assert (!ms_ole_stat (&stat, ole, "/", STREAM_NAME));
+	g_assert (stat.type == MsOleStreamT);
+	g_assert (stat.size == BLOCK_LEN * NUM_BLOCKS);
+
+	g_print ("\n\nre-read tests\n");
+	g_assert (!ms_ole_stream_open (&stream, ole, "/", STREAM_NAME, 'r'));
+	hammer_stream (stream, FALSE);
+	g_assert (!ms_ole_stream_close (&stream));
+	ms_ole_debug (ole, 1);
+
+	ms_ole_destroy (&ole);
+	g_print ("\nPassed\n\n");
+}
+
 int
 main (int argc, char **argv)
 {
@@ -883,6 +998,11 @@ main (int argc, char **argv)
 	if (argc<2)
 		syntax_error(0);
 
+	if (!g_strcasecmp (argv [1], "regression")) {
+		do_regression_tests ();
+		return 0;
+	}
+
 	printf ("Ole file '%s'\n", argv[1]);
 	if (ms_ole_open_vfs (&ole, argv[1], TRUE, NULL)
 	    != MS_OLE_ERR_OK) {
@@ -954,15 +1074,15 @@ main (int argc, char **argv)
 			ms_ole_debug (ole, 2);
 		else if (g_strcasecmp (ptr, "vba") == 0)
 			dump_vba (ole);
-		else if (g_strcasecmp (ptr,"help") == 0 ||
-			 g_strcasecmp (ptr,"?") == 0 ||
-			 g_strcasecmp (ptr,"info") == 0 ||
-			 g_strcasecmp (ptr,"man") == 0)
+		else if (g_strcasecmp (ptr, "help") == 0 ||
+			 g_strcasecmp (ptr, "?") == 0 ||
+			 g_strcasecmp (ptr, "info") == 0 ||
+			 g_strcasecmp (ptr, "man") == 0)
 			list_commands ();
-		else if (g_strcasecmp (ptr,"exit") == 0 ||
-			 g_strcasecmp (ptr,"quit") == 0 ||
-			 g_strcasecmp (ptr,"q") == 0 ||
-			 g_strcasecmp (ptr,"bye") == 0)
+		else if (g_strcasecmp (ptr, "exit") == 0 ||
+			 g_strcasecmp (ptr, "quit") == 0 ||
+			 g_strcasecmp (ptr, "q") == 0 ||
+			 g_strcasecmp (ptr, "bye") == 0)
 			exit = 1;
 	}
 	while (!exit && interact);
